//  Diffusion Client Library for iOS, OS X and tvOS
//
//  Copyright (c) 2015, 2016 Push Technology Ltd., All Rights Reserved.
//
//  Use is subject to license terms.
//
//  NOTICE: All information contained herein is, and remains the
//  property of Push Technology. The intellectual and technical
//  concepts contained herein are proprietary to Push Technology and
//  may be covered by U.S. and Foreign Patents, patents in process, and
//  are protected by trade secret or copyright law.

@import Foundation;
#import <Diffusion/PTDiffusionStreamDelegate.h>

@class PTDiffusionContent;
@class PTDiffusionReceiveContext;

NS_ASSUME_NONNULL_BEGIN

/**
 @brief The message stream delegate protocol defines the methods to be implemented by classes
 wishing to receive messages.

 @see PTDiffusionMessagingFeature

 @since 5.6
 */
@protocol PTDiffusionMessageStreamDelegate <PTDiffusionStreamDelegate>

@required

/**
 A message was received for a topic path handled by the topic stream.

 @param stream The message stream that received the message.

 @param topicPath The topic path on which the message was received.

 @param content The content of the message.

 @param context The message context.

 @since 5.6
 */
-(void)          diffusionStream:(PTDiffusionStream *)stream
    didReceiveMessageOnTopicPath:(NSString *)topicPath
                         content:(PTDiffusionContent *)content
                         context:(PTDiffusionReceiveContext *)context;

@end

NS_ASSUME_NONNULL_END
