//  Diffusion Client Library for iOS, OS X and tvOS
//
//  Copyright (c) 2015 Push Technology Ltd., All Rights Reserved.
//
//  Use is subject to license terms.
//
//  NOTICE: All information contained herein is, and remains the
//  property of Push Technology. The intellectual and technical
//  concepts contained herein are proprietary to Push Technology and
//  may be covered by U.S. and Foreign Patents, patents in process, and
//  are protected by trade secret or copyright law.

@import Foundation;
#import <Diffusion/PTDiffusionTopicType.h>

@class PTDiffusionFieldMetadata;
@class PTDiffusionPagedRecordTopicOrderKey;
@class PTDiffusionPagedTopicDuplicates;
@class PTDiffusionPagedTopicOrder;
@class PTDiffusionPagedTopicRules;
@class PTDiffusionRecordContentSchema;
@class PTDiffusionRecordMetadata;
@class PTDiffusionTopicAttributes;
@class PTDiffusionTopicSchema;

@protocol PTDiffusionCustomTopicDetailsBuilder;
@protocol PTDiffusionPagedRecordTopicDetailsBuilder;
@protocol PTDiffusionPagedStringTopicDetailsBuilder;
@protocol PTDiffusionRecordTopicDetailsBuilder;
@protocol PTDiffusionRoutingTopicDetailsBuilder;
@protocol PTDiffusionSingleValueTopicDetailsBuilder;
@protocol PTDiffusionSlaveTopicDetailsBuilder;
@protocol PTDiffusionTopicDetailsBuilder;
@protocol PTDiffusionTopicNotifyTopicDetailsBuilder;

NS_ASSUME_NONNULL_BEGIN

/**
 Name of exception raised when topic details cannot be created due to incomplete or incorrect
 builder configuration.
 
 @since 5.7
 */
extern NSString *const PTDiffusionInvalidTopicDetailsBuilderStateException;

/**
 Name of exception raised when topic details cannot be created due to lack of support in
 this client library for the topic type.

 @since 5.7
 */
extern NSString *const PTDiffusionUnsupportedTopicDetailsBuildException;

/**
 The signature required for a builder block accepting a custom topic details builder.

 @deprecated Since 5.9. Will be removed in a future release.

 @since 5.7
 */
__deprecated_msg("Will be removed in a future release.")
typedef void (^PTDiffusionCustomTopicDetailsBuilderBlock)(id<PTDiffusionCustomTopicDetailsBuilder> builder);

/**
 The signature required for a builder block accepting a paged record topic details builder.

 @deprecated Since 5.9. Will be removed in a future release.

 @since 5.7
 */
__deprecated_msg("Will be removed in a future release.")
typedef void (^PTDiffusionPagedRecordTopicDetailsBuilderBlock)(id<PTDiffusionPagedRecordTopicDetailsBuilder> builder);

/**
 The signature required for a builder block accepting a paged string topic details builder.

 @deprecated Since 5.9. Will be removed in a future release.

 @since 5.7
 */
__deprecated_msg("Will be removed in a future release.")
typedef void (^PTDiffusionPagedStringTopicDetailsBuilderBlock)(id<PTDiffusionPagedStringTopicDetailsBuilder> builder);

/**
 The signature required for a builder block accepting a record topic details builder.

 @since 5.7
 */
typedef void (^PTDiffusionRecordTopicDetailsBuilderBlock)(id<PTDiffusionRecordTopicDetailsBuilder> builder);

/**
 The signature required for a builder block accepting a routing topic details builder.

 @since 5.7
 */
typedef void (^PTDiffusionRoutingTopicDetailsBuilderBlock)(id<PTDiffusionRoutingTopicDetailsBuilder> builder);

/**
 The signature required for a builder block accepting a single value topic details builder.

 @since 5.7
 */
typedef void (^PTDiffusionSingleValueTopicDetailsBuilderBlock)(id<PTDiffusionSingleValueTopicDetailsBuilder> builder);

/**
 The signature required for a builder block accepting a slave topic details builder.

 @since 5.7
 */
typedef void (^PTDiffusionSlaveTopicDetailsBuilderBlock)(id<PTDiffusionSlaveTopicDetailsBuilder> builder);

/**
 The signature required for a builder block accepting a basic topic details builder.

 @since 5.7
 */
typedef void (^PTDiffusionTopicDetailsBuilderBlock)(id<PTDiffusionTopicDetailsBuilder> builder);

/**
 The signature required for a builder block accepting a topic notify topic details builder.

 @deprecated Since 5.9. Will be removed in a future release.

 @since 5.7
 */
__deprecated_msg("Will be removed in a future release.")
typedef void (^PTDiffusionTopicNotifyTopicDetailsBuilderBlock)(id<PTDiffusionTopicNotifyTopicDetailsBuilder> builder);

/**
 @brief Topic details describe a topic.
 
 Where many topics are to have the same definition, the same topic detail can (and indeed should)
 be used to create many topics.

 Topic details are also used to provide details of existing topics to clients.
 In this case schema and/or attributes may not be present in the details if that level of detail
 was not requested.
 
 @see PTDiffusionTopicsFeature
 @see PTDiffusionTopicControlFeature

 @since 5.6
 */
@interface PTDiffusionTopicDetails : NSObject <NSCopying>

+(instancetype)new NS_UNAVAILABLE;

-(instancetype)init NS_UNAVAILABLE;

/**
 Returns new topic details of the given type set with all default settings.
 
 This provides a simpler mechanism than using a builder where all of the builder's default
 values are suitable and there are no mandatory properties.

 @param type The topic type for which default details are to be returned.

 @exception PTDiffusionUnsupportedTopicDetailsBuildException If the creation of topic details
 for the given type is not supported by this client library.

 @exception PTDiffusionInvalidTopicDetailsBuilderStateException If default details can't be
 created for the specified type as a builder has to be used.
 
 @since 5.7
 */
+(instancetype)topicDetailsWithType:(PTDiffusionTopicType)type;

/**
 Returns new topic details for a child list topic with all default settings.

 @deprecated Since 5.9. Will be removed in a future release.

 @since 5.7
 */
+(instancetype)childListTopicDetails __deprecated_msg("Will be removed in a future release.");

/**
 Returns new topic details for a child list topic configured using the given builder block which
 is supplied an object conforming to the PTDiffusionTopicDetailsBuilder protocol.

 @deprecated Since 5.9. Will be removed in a future release.

 @since 5.7
 */
+(instancetype)childListTopicDetailsWithBuilderBlock:(PTDiffusionTopicDetailsBuilderBlock)block __deprecated_msg("Will be removed in a future release.");

/**
 Returns new topic details for a custom topic configured using the given builder block which
 is supplied an object conforming to the PTDiffusionCustomTopicDetailsBuilder protocol.

 @deprecated Since 5.9. Will be removed in a future release.

 @since 5.7
 */
+(instancetype)customTopicDetailsWithBuilderBlock:(PTDiffusionCustomTopicDetailsBuilderBlock)block __deprecated_msg("Will be removed in a future release.");

/**
 Returns new topic details for a paged record topic with all default settings.

 @deprecated Since 5.9. Will be removed in a future release.

 @since 5.7
 */
+(instancetype)pagedRecordTopicDetails __deprecated_msg("Will be removed in a future release.");

/**
 Returns new topic details for a paged record topic configured using the given builder block which
 is supplied an object conforming to the PTDiffusionPagedRecordTopicDetailsBuilder protocol.

 @deprecated Since 5.9. Will be removed in a future release.

 @since 5.7
 */
+(instancetype)pagedRecordTopicDetailsWithBuilderBlock:(PTDiffusionPagedRecordTopicDetailsBuilderBlock)block __deprecated_msg("Will be removed in a future release.");

/**
 Returns new topic details for a paged string topic with all default settings.

 @deprecated Since 5.9. Will be removed in a future release.

 @since 5.7
 */
+(instancetype)pagedStringTopicDetails __deprecated_msg("Will be removed in a future release.");

/**
 Returns new topic details for a paged string topic configured using the given builder block which
 is supplied an object conforming to the PTDiffusionPagedStringTopicDetailsBuilder protocol.

 @deprecated Since 5.9. Will be removed in a future release.

 @since 5.7
 */
+(instancetype)pagedStringTopicDetailsWithBuilderBlock:(PTDiffusionPagedStringTopicDetailsBuilderBlock)block __deprecated_msg("Will be removed in a future release.");

/**
 Returns new topic details for a record topic with all default settings.

 @since 5.7
 */
+(instancetype)recordTopicDetails;

/**
 Returns new topic details for a record topic configured using the given builder block which
 is supplied an object conforming to the PTDiffusionRecordTopicDetailsBuilder protocol.

 @since 5.7
 */
+(instancetype)recordTopicDetailsWithBuilderBlock:(PTDiffusionRecordTopicDetailsBuilderBlock)block;

/**
 Returns new topic details for a routing topic with all default settings.

 @since 5.7
 */
+(instancetype)routingTopicDetails;

/**
 Returns new topic details for a routing topic configured using the given builder block which
 is supplied an object conforming to the PTDiffusionRoutingTopicDetailsBuilder protocol.

 @since 5.7
 */
+(instancetype)routingTopicDetailsWithBuilderBlock:(PTDiffusionRoutingTopicDetailsBuilderBlock)block;

/**
 Returns new topic details for a single value topic with all default settings.

 @since 5.7
 */
+(instancetype)singleValueTopicDetails;

/**
 Returns new topic details for a single value topic configured using the given builder block which
 is supplied an object conforming to the PTDiffusionSingleValueTopicDetailsBuilder protocol.

 @since 5.7
 */
+(instancetype)singleValueTopicDetailsWithBuilderBlock:(PTDiffusionSingleValueTopicDetailsBuilderBlock)block;

/**
 Returns new topic details for a slave topic configured using the given builder block which
 is supplied an object conforming to the PTDiffusionSlaveTopicDetailsBuilder protocol.

 @since 5.7
 */
+(instancetype)slaveTopicDetailsWithBuilderBlock:(PTDiffusionSlaveTopicDetailsBuilderBlock)block;

/**
 Returns new topic details for a stateless topic with all default settings.

 @since 5.7
 */
+(instancetype)statelessTopicDetails;

/**
 Returns new topic details for a stateless topic configured using the given builder block which
 is supplied an object conforming to the PTDiffusionTopicDetailsBuilder protocol.

 @since 5.7
 */
+(instancetype)statelessTopicDetailsWithBuilderBlock:(PTDiffusionTopicDetailsBuilderBlock)block;

/**
 Returns new topic details for a topic notify topic with all default settings.

 @deprecated Since 5.9. Will be removed in a future release.

 @since 5.7
 */
+(instancetype)topicNotifyTopicDetails __deprecated_msg("Will be removed in a future release.");

/**
 Returns new topic details for a topic notify topic configured using the given builder block which
 is supplied an object conforming to the PTDiffusionTopicNotifyTopicDetailsBuilder protocol.

 @deprecated Since 5.9. Will be removed in a future release.

 @since 5.7
 */
+(instancetype)topicNotifyTopicDetailsWithBuilderBlock:(PTDiffusionTopicNotifyTopicDetailsBuilderBlock)block __deprecated_msg("Will be removed in a future release.");

/**
 Attributes associated with this topic.
 
 @since 5.7
 */
@property(nullable, nonatomic, readonly) PTDiffusionTopicAttributes* attributes;

/**
 Schema associated with this topic.
 
 @since 5.7
 */
@property(nullable, nonatomic, readonly) PTDiffusionTopicSchema* schema;

/**
 The type of topic.

 @since 5.6
 */
@property(nonatomic, readonly) PTDiffusionTopicType type;

/**
 Compares the receiver to the given topic details.

 @param topicDetails The topic details object with which to compare the receiver.

 @return `YES` if the topic details is equal to the receiver, otherwise `NO`.
 
 @since 5.7
 */
-(BOOL)isEqualToTopicDetails:(nullable PTDiffusionTopicDetails *)topicDetails;

@end

/**
 @brief All topic details builders conform to this protocol.
 
 @see PTDiffusionTopicDetails

 @since 5.7
 */
@protocol PTDiffusionTopicDetailsBuilder <NSObject>

/**
 The topic reference or a zero length string if not set.

 @exception NSInvalidArgumentException On setting the property,
 if the supplied reference is `nil`.

 @since 5.7
 */
@property(nonatomic, copy) NSString* reference;

/**
 The 'tidy on unsubscribe' flag.

 By default, if a client unsubscribes from a topic, it may still receive
 content already queued for it on that topic.

 If this option is set to `YES`, when a client unsubscribes from a topic then
 any content for that topic that is still queued for the client is removed.
 There can be a performance overhead to using this option as the client queue
 must be locked whilst content is removed, however it may prove useful for
 preventing unwanted data being sent to clients.

 @since 5.7
 */
@property(nonatomic) BOOL tidiesOnUnsubscribe;

/**
 Sets a topic property. This is used for setting restricted or legacy topic properties.

 @param value The property value.

 @param key The property key. The values for these keys and permitted valuers are
 published in the user documentation.

 @exception NSInvalidArgumentException If either value or key is `nil`, or the
 key is not supported.

 @note Properties may be removed with the removePropertyForKey: method.

 @since 5.7
 */
-(void)setPropertyValue:(NSString *)value forKey:(NSString *)key;

/**
 Removes (unsets) a topic property.

 @param key The property key.

 @return `YES` if the key was set and has now been unset.
 
 @exception NSInvalidArgumentException If key is `nil`.

 @since 5.7
 */
-(BOOL)removePropertyForKey:(NSString *)key;

/**
 Gets the value of a topic property.

 @param key The property key. The values for these keys and permitted valuers are
 published in the user documentation.

 @exception NSInvalidArgumentException If key is `nil`.

 @since 5.7
 */
-(nullable NSString *)propertyValueForKey:(NSString *)key;

/**
 An immutable snapshot of the topic properties.

 @since 5.7
 */
@property(nonatomic, readonly) NSDictionary<NSString *, NSString *> * properties;

@end

/**
 @brief The protocol to which custom topic details builders conform.

 @see PTDiffusionTopicDetails

 @deprecated Since 5.9. Will be removed in a future release.

 @since 5.7
 */
__deprecated_msg("Will be removed in a future release.")
@protocol PTDiffusionCustomTopicDetailsBuilder <PTDiffusionTopicDetailsBuilder>

/**
 Specifies the full path name of the CustomTopicDataHandler class available at
 the server. An instance of this class will be instantiated to represent the
 topic and maintain its state.
 
 @exception NSInvalidArgumentException On setting this property, if the supplied
 handler is `nil` or empty (zero length).

 @since 5.7
 */
@property(nonatomic, copy) NSString* handler;

@end

/**
 @brief The protocol to which record topic details builders conform.

 @see PTDiffusionTopicDetails

 @since 5.7
 */
@protocol PTDiffusionRecordTopicDetailsBuilder <PTDiffusionTopicDetailsBuilder>

/**
 The record content schema defining the layout of stored record content values
 for this topic.

 @exception NSInvalidArgumentException On setting this property with a value of `nil`.

 @since 5.7
 */
@property(nonatomic, copy) PTDiffusionRecordContentSchema* metadata;

/**
 The value defining empty field.

 @exception NSInvalidArgumentException On setting this property with a value of `nil`.

 @since 5.7
 */
@property(nonatomic, copy) NSString* emptyFieldValue;

@end

/**
 @brief The protocol to which single value topic details builders conform.

 @see PTDiffusionTopicDetails

 @since 5.7
 */
@protocol PTDiffusionSingleValueTopicDetailsBuilder <PTDiffusionTopicDetailsBuilder>

/**
 The field metadata defining the format of stored field values for this topic.
 
 @exception NSInvalidArgumentException On setting this property with a value of `nil`.

 @since 5.7
 */
@property(nonatomic, copy) PTDiffusionFieldMetadata* metadata;

@end

/**
 @brief All paged topic details builders conform to this protocol.

 @see PTDiffusionTopicDetails

 @deprecated Since 5.9. Will be removed in a future release.

 @since 5.7
 */
__deprecated_msg("Will be removed in a future release.")
@protocol PTDiffusionPagedTopicDetailsBuilder <PTDiffusionTopicDetailsBuilder>

/**
 Specifies comparator ordering is to be applied to the topic.
 
 This allows ordering to be specified which will be handled by a class at the server
 which is used to determine the order.

 The comparator name should be the full class name of a Java class available at the
 server which is either of generic type String or Record depending upon the subtype.
 An instance of this class will be instantiated at the server for the topic to use
 for determining the line order.

 Calling this method would replace any previously specified ordering policy.

 A duplicates policy of 'not allowed' is assumed.
 
 @param comparatorName The full class name of the comparator.

 @exception NSInvalidArgumentException if comparatorName is `nil` or of zero length.

 @since 5.7
 */
-(void)orderWithComparatorName:(NSString *)comparatorName;

/**
 Specifies comparator ordering is to be applied to the topic.

 This allows ordering to be specified which will be handled by a class at the server
 which is used to determine the order.

 The comparator name should be the full class name of a Java class available at the
 server which is either of generic type String or Record depending upon the subtype.
 An instance of this class will be instantiated at the server for the topic to use
 for determining the line order.
 
 Calling this method would replace any previously specified ordering policy.
 
 @param duplicatesPolicy The duplicate policy to use.

 @param comparatorName The full class name of the comparator.
 
 @exception NSInvalidArgumentException if duplicatesPolicy is `nil`, or
 comparatorName is `nil` or of zero length.

 @since 5.7
 */
-(void)orderWithDuplicatesPolicy:(PTDiffusionPagedTopicDuplicates *)duplicatesPolicy
                  comparatorName:(NSString *)comparatorName;

/**
 This specifies that the topic ordering should be set to unordered.

 This is the default mode but this method may be used to remove any ordering
 policy previously specified.

 @since 5.7
 */
-(void)unordered;

@end

/**
 @brief The protocol to which paged record topic details builders conform.

 @see PTDiffusionTopicDetails

 @deprecated Since 5.9. Will be removed in a future release.

 @since 5.7
 */
__deprecated_msg("Will be removed in a future release.")
@protocol PTDiffusionPagedRecordTopicDetailsBuilder <PTDiffusionPagedTopicDetailsBuilder>

/**
 The record metadata defining the layout of stored record values for this topic.

 @since 5.7
 */
@property(nonatomic, copy) PTDiffusionRecordMetadata* metadata;

/**
 Specifies that the topic should be ordered by policy, defining the policy in terms of
 a list of order keys.

 Fields specified for sorting will be sorted according to their type.
 Strings will be sorted according to default collation rules (unless additional rules
 are applied in the key) and numeric types will be sorted according to the magnitude
 of the number.
 
 Calling this method replaces any previously declared ordering policy previously set
 on the builder.

 A duplicates policy of 'not allowed' is assumed.
 
 @param orderKeys The order keys that specify how to sort the topic data.
 Each key much specify a different field in the metadata in the sort priority order
 required. Each key must specify a different metadata field.
 If no keys are supplied, it is assumed that sorting will be applied to all fields
 in the order they occur within the record and in ascending order with default
 string collation.
 
 @exception NSInvalidArgumentException If orderKeys is `nil`.

 @since 5.7
 */
-(void)orderWithOrderKeys:(NSArray<PTDiffusionPagedRecordTopicOrderKey *> *)orderKeys;

/**
 Specifies that the topic should be ordered by policy, defining the policy in terms of
 a list of order keys.

 Fields specified for sorting will be sorted according to their type.
 Strings will be sorted according to default collation rules (unless additional rules
 are applied in the key) and numeric types will be sorted according to the magnitude
 of the number.

 Calling this method replaces any previously declared ordering policy previously set
 on the builder.
 
 @param duplicatesPolicy Specifies the duplicates policy to be applied when ordering
 determines that two records are equal.

 @param orderKeys The order keys that specify how to sort the topic data.
 Each key much specify a different field in the metadata in the sort priority order
 required. Each key must specify a different metadata field.
 If no keys are supplied, it is assumed that sorting will be applied to all fields
 in the order they occur within the record and in ascending order with default
 string collation
 
 @exception NSInvalidArgumentException If duplicatesPolicy or orderKeys are `nil`.
 
 @since 5.7
 */
-(void)orderWithDuplicatesPolicy:(PTDiffusionPagedTopicDuplicates *)duplicatesPolicy
                       orderKeys:(NSArray<PTDiffusionPagedRecordTopicOrderKey *> *)orderKeys;

@end

/**
 @brief The protocol to which paged string topic details builders conform.

 @see PTDiffusionTopicDetails

 @deprecated Since 5.9. Will be removed in a future release.

 @since 5.7
 */
__deprecated_msg("Will be removed in a future release.")
@protocol PTDiffusionPagedStringTopicDetailsBuilder <PTDiffusionPagedTopicDetailsBuilder>

/**
 Specifies that the topic should be ordered by policy.

 The policy would be ascending order with default collation and a duplicates policy
 of 'not allowed'.

 Calling this method replaces any previously declared ordering policy.

 @since 5.7
 */
-(void)order;

/**

 Specifies that the topic should be ordered by policy.

 The order and collation rules may be specified.

 Calling this method replaces any previously declared ordering policy.
 
 @param duplicatesPolicy Specifies the policy to be applied when two lines are determined
 equal according to the policy.
 
 @param order The order.
 
 @param rules The collation rules to apply. If `nil` then 'no rules' is assumed.
 
 @exception NSInvalidArgumentException if duplicatesPolicy or order are `nil`.

 @since 5.7
 */
-(void)orderWithDuplicatesPolicy:(PTDiffusionPagedTopicDuplicates *)duplicatesPolicy
                           order:(PTDiffusionPagedTopicOrder *)order
                           rules:(nullable PTDiffusionPagedTopicRules *)rules;

@end

/**
 @brief The protocol to which routing topic details builders conform.

 @see PTDiffusionTopicDetails

 @since 5.7
 */
@protocol PTDiffusionRoutingTopicDetailsBuilder <PTDiffusionTopicDetailsBuilder>

/**
 Sets the routing handler class name if routing is to be performed at the server.

 If routing is not to be handled at the server, it is expected that all subscription
 requests to the topic will be handled via a client using the subscription
 control feature.

 If not explicitly set, the value will be a zero length string indicating that
 routing is not handled by the server.
 
 @exception NSInvalidArgumentException On setting this property with a value of `nil`.

 @since 5.7
 */
@property(nonatomic, copy) NSString* handler;

@end

/**
 @brief The protocol to which slave topic details builders conform.

 @see PTDiffusionTopicDetails

 @since 5.7
 */
@protocol PTDiffusionSlaveTopicDetailsBuilder <PTDiffusionTopicDetailsBuilder>

/**
 The master topic path.
 
 This must identify an existing topic that is stateful and is not another slave topic.
 
 @exception NSInvalidArgumentException On setting this property with a value of `nil`.

 @since 5.7
 */
@property(nonatomic, copy) NSString* masterTopicPath;

@end

/**
 @brief The protocol to which topic notify topic details builders conform.

 @see PTDiffusionTopicDetails

 @deprecated Since 5.9. Will be removed in a future release.

 @since 5.7
 */
__deprecated_msg("Will be removed in a future release.")
@protocol PTDiffusionTopicNotifyTopicDetailsBuilder <PTDiffusionTopicDetailsBuilder>

/**
 Sets the 'metadata caching' option.

 If not explicitly set, it is assumed that metadata caching is used therefore this
 only needs to be called to turn the option off in situations where metadata names
 are not unique.
 
 Caching can significantly improve performance and reduce bandwidth required for
 notifications from such a topic but it can only be used if all instances of metadata
 used by topics that get notified (potentially all) have unique names.
 Using caching when names are not unique can lead to incorrect metadata notifications.

 @since 5.7
 */
@property(nonatomic) BOOL cachesMetadata;

@end

NS_ASSUME_NONNULL_END
