//  Diffusion Client Library for iOS, OS X and tvOS
//
//  Copyright (c) 2015, 2016 Push Technology Ltd., All Rights Reserved.
//
//  Use is subject to license terms.
//
//  NOTICE: All information contained herein is, and remains the
//  property of Push Technology. The intellectual and technical
//  concepts contained herein are proprietary to Push Technology and
//  may be covered by U.S. and Foreign Patents, patents in process, and
//  are protected by trade secret or copyright law.

@import Foundation;

/**
 Indicates the type of update represented by the content which may vary according to topic type.

 @since 5.6
 */
typedef NS_ENUM(NSUInteger, PTDiffusionUpdateType) {
    /**
     The content represents a delta of change to the topic state.

     @since 5.6
     */
    PTDiffusionUpdateType_Snapshot = 1,

    /**
     The content is a complete representation of topic state.

     @since 5.6
     */
    PTDiffusionUpdateType_Delta = 2,
};

NS_ASSUME_NONNULL_BEGIN

/**
 Returns a string representation of the given update type.

 @param updateType The update type for which a description is to be returned.
 
 @since 5.7
 */
NSString* PTDiffusionUpdateTypeToString(PTDiffusionUpdateType updateType);

NS_ASSUME_NONNULL_END
